from setuptools import setup, find_packages

setup(
    name='a5_client_utils',
    version='0.1.2',
    packages=find_packages(),
    description='Auxiliary functions for a5 API clients',
    author='Juan F. Bianchi',
    author_email='jbianchi@ina.gob.ar',
    url='https://github.com/jbianchi81/a5-client-utils',  # Optional
)
